﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Transactions;
using HeuristicLab.Services.Hive.Common;

namespace HeuristicLab.Services.Hive.DataAccess {
  public class TransactionManager {
    public void UseTransaction(Action call, bool serializable = false, bool longRunning = false) {
      TransactionScope transaction = CreateTransaction(serializable, longRunning);
      try {
        call();
        transaction.Complete();
      }
      finally {
        transaction.Dispose();
      }
    }

    public T UseTransaction<T>(Func<T> call, bool serializable = false, bool longRunning = false) {
      TransactionScope transaction = CreateTransaction(serializable, longRunning);
      try {
        T result = call();
        transaction.Complete();
        return result;
      }
      finally {
        transaction.Dispose();
      }
    }
    
    private static TransactionScope CreateTransaction(bool serializable, bool longRunning) {
      var options = new TransactionOptions();
      if (serializable)
        options.IsolationLevel = IsolationLevel.Serializable;
      else
        options.IsolationLevel = IsolationLevel.ReadUncommitted;

      if (longRunning)
        options.Timeout = ApplicationConstants.LongRunningDatabaseCommandTimeout;
      
      return new TransactionScope(TransactionScopeOption.Required, options);
    }
  }
}
